﻿//                             ___           ___         ___     
//  Feel free to use and      /\  \         /\  \       /\__\    
//  modify any of this code   \:\  \       /::\  \     /:/  /    
//                             \:\  \     /:/\:\__\   /:/  /     
//   ________  ________    _____\:\  \   /:/ /:/  /  /:/  /  ___ 
//  |\   __  \|\   __  \  /::::::::\__\ /:/_/:/  /  /:/__/  /\__\
//  \ \  \|\  \ \  \|\  \ \:\~~\~~\/__/ \:\/:/  /   \:\  \ /:/  /
//   \ \   ____\ \   ____\ \:\  \        \::/__/     \:\  /:/  / 
//    \ \  \___|\ \  \___|  \:\  \        \:\  \      \:\/:/  /  
//     \ \__\    \ \__\      \:\__\        \:\__\      \::/  /   
//      \|__|     \|__|       \/__/         \/__/       \/__/     
//
//
using UnityEngine;
using UnityEngine.Events;
using System.Linq;
using System.Collections.Generic;
using System.Collections;

namespace PPnpc
{
	public class NpcGadget : MonoBehaviour
	{
		public Gadgets Gadget;
		public LightSprite[] Lights;
		public Coroutine AnimateRoutine;
		public bool BeenActivated = false;
		public PersonBehaviour TargetPerson;
		public float Timer1;
		public float Timer2;
		public float[] MiscFloats;
		public string[] MiscStrings;
		public bool[] MiscBools;
		public Transform TargetTrans;
		public SpriteRenderer[] SRS;
		public AudioClip[] AudioClips;
		public Color[] MiscColors;
		private AudioSource audioSource;
		private int _teamId = 0;
		private bool AlreadyAI = false;
		public Dictionary<string, Rigidbody2D>   RB  = new Dictionary<string, Rigidbody2D>();
		public Dictionary<string, LimbBehaviour> LB  = new Dictionary<string, LimbBehaviour>();
		public float Facing;
		public GameObject AIgo;
		public GameObject MiscGameObject;
		public PhysicalBehaviour PB;
		public Coroutine xCoroutine;
		public NpcGadget Expansion;
		public bool IsChip = false;
		public int _colliderId = 0;
		public Collider2D connector;
		public NpcGadget ChildGadget  = null;
		public NpcGadget ParentGadget = null;
		public float ConnectCoolDown = 0f;
		public DistanceJoint2D spring;

		

		

		public int TeamId { 
			get { 
				return Mathf.Clamp(_teamId, 0, NpcGlobal.MaxTeamId); 
				} 
			set { 
				if (value > NpcGlobal.MaxTeamId) value = 0; 
				_teamId = value;
				if (_teamId > 0)
				{
					string teamColor = xxx.GetTeamColor(_teamId);
					
					if ( Gadget == Gadgets.AIChip )
					{
						if (ColorUtility.TryParseHtmlString(teamColor + "FF", out Color color))  MiscColors[0] = color;
						if (ColorUtility.TryParseHtmlString(teamColor + "99", out Color color2)) MiscColors[1] = color2;
					}
					else if (Gadget == Gadgets.Rebirther )
					{
						string hexalpha = Mathf.RoundToInt(Mathf.Clamp(PB.charge * 2, 25, 255)).ToString("X2");
						if (ColorUtility.TryParseHtmlString(teamColor + hexalpha, out Color color))  MiscColors[0] = color;
						hexalpha = Mathf.RoundToInt(Mathf.Clamp(PB.charge , 15, 125)).ToString("X2");
						if (ColorUtility.TryParseHtmlString(teamColor + hexalpha, out Color color2)) MiscColors[1] = color2;
					}

					xxx.CheckRebirthers();

				}
			} 
		}

		public void Init( Gadgets _gadget )
		{
			Gadget = _gadget;

			switch ( Gadget )
			{
				case Gadgets.AIChip:
					IsChip = false;
					SetupAIChip();
					AddBoxTop();
					break;

				case Gadgets.Memory:
				case Gadgets.Firearms:
				case Gadgets.Karate:
				case Gadgets.Melee:
					SRS = new SpriteRenderer[1];
                    SRS[0] = gameObject.GetComponent<SpriteRenderer>();
					AddBoxTop();
					PB     = gameObject.GetComponent<PhysicalBehaviour>();
					IsChip = true;
					break;

				case Gadgets.Expansion:
					SetupExpansion();
					break;
			}
		}


		public void AddBoxTop()
        {
            EdgeCollider2D edge = gameObject.AddComponent<EdgeCollider2D>();
            SpriteRenderer sr   = gameObject.GetComponent<SpriteRenderer>();
            edge.isTrigger      = true;
			
			float y             = (sr.bounds.max.y - sr.bounds.min.y) / 2f;
			float x             = (sr.bounds.max.x - sr.bounds.min.x) / 2f;
            edge.points         = new Vector2[]
            {
				new Vector2(-x, y),
				new Vector2(x, y),
            };
            edge.enabled = true;
            _colliderId = edge.GetHashCode();
			connector = edge;
        }


		public void Shot(Shot shot)
		{
			
			Damaged();
		}
		

		public void Damaged()
		{
			if ( Gadget == Gadgets.AIChip ) { 
				List<Collider2D> colliders = new List<Collider2D>();
				ModAPI.CreateParticleEffect("IonExplosion", transform.position);
				Physics2D.OverlapBox(transform.position, new Vector2(10,5),0f, new ContactFilter2D(), colliders);
				foreach ( Collider2D collider in colliders )
				{
					if ( collider.gameObject.TryGetComponent<PhysicalBehaviour>( out PhysicalBehaviour phys ) )
					{
						phys.SendMessage("OnEMPHit", SendMessageOptions.DontRequireReceiver);
					}
				}
				GameObject.Destroy(this.gameObject);
			}
			else if ( Gadget == Gadgets.Karate )
            {
				List<Collider2D> colliders = new List<Collider2D>();
				ModAPI.CreateParticleEffect("IonExplosion", transform.position);
				Physics2D.OverlapBox(transform.position, new Vector2(10,5),0f, new ContactFilter2D(), colliders);
				foreach ( Collider2D collider in colliders )
				{
					if ((bool)(UnityEngine.Object)collider.attachedRigidbody)
					{
						Vector3 vector3 = collider.transform.position - transform.position;
						float num2      = (float)((-(double)vector3.magnitude + (double)3) * ((double)4 / (double)3));

						collider.attachedRigidbody.AddForce((Vector2)(num2 * vector3.normalized), ForceMode2D.Impulse);
					}
				}
				GameObject.Destroy(this.gameObject);

			}
			else if ( Gadget == Gadgets.Firearms )
            {
				List<Collider2D> colliders = new List<Collider2D>();
				
				//ModAPI.CreateParticleEffect("RedBarrelExplosion", transform.position);
				ExplosionCreator.CreateFragmentationExplosion(new ExplosionCreator.ExplosionParameters()
                {
					Position                = transform.position,
					FragmentationRayCount   = 30u,
					BallisticShrapnelCount  = 5,
					CreateParticlesAndSound = true,
					FragmentForce           = 3,
					Range                   = 6
                });

				GameObject.Destroy(this.gameObject);

			}
			else if ( IsChip )
            {
                ModAPI.CreateParticleEffect( "IonExplosion", transform.position );
                GameObject.Destroy( this.gameObject );
            }
		}

		public void Use(ActivationPropagation activation)
		{
			
			if ( Gadget == Gadgets.AIChip )
			{
				audioSource.PlayOneShot(AudioClips[0], 2f);
				TeamId += 1;
				string teamColor = xxx.GetTeamColor(TeamId);
				MiscStrings[0] = teamColor;
				MiscStrings[1] = teamColor;
			}


			
		}


		void Awake()
		{
			audioSource              = gameObject.AddComponent<AudioSource>();
			audioSource.spread       = 35f;
			audioSource.volume       = 1f;
			audioSource.minDistance  = 15f;
			audioSource.spatialBlend = 1f;
			audioSource.dopplerLevel = 0f;
		}

		

		public void SetupExpansion()
		{
			MiscBools = new bool[] {false, false};
		}

		

		
		
		

		public void SetupAIChip()
		{
			//	Add fekn lights
			Lights = new LightSprite[2];

			Lights[0]            = ModAPI.CreateLight(transform, Color.Lerp(new Color32(0,0,50,255), new Color32(0,0,255,255),Time.fixedDeltaTime * 100));
			Lights[0].Brightness = 1.5f;
			Lights[0].Radius     = 0.225f;
			Lights[0].transform.SetParent(this.gameObject.transform);

			
			Lights[1]            = ModAPI.CreateLight(transform, Color.Lerp(new Color32(50,0,0,255), new Color32(255,0,0,255),Time.fixedDeltaTime * 100));
			
			Lights[1].Brightness = 0.5f;
			Lights[1].Radius     = 0.225f;
			Lights[1].transform.SetParent(this.gameObject.transform);

			SRS = new SpriteRenderer[2];
			
			GameObject TeamSelect = gameObject.transform.Find("TeamSelect").gameObject;
			SRS[1] = TeamSelect.GetComponent<SpriteRenderer>();



			MiscFloats = new float[]{ Time.time };
			MiscStrings = new string[]
			{
				"000000", "000000"
			};


			MiscColors = new Color[] { Color.red, Color.green, Color.blue };

			PB.ForceNoCharge = false;
			PB.ChargeBurns = true;
			
			AnimateRoutine = StartCoroutine(IAnimateAIChip());

		}

		public IEnumerator IAnimateMiscChip()
		{
			ModAPI.CreateParticleEffect("IonExplosion", transform.position);
			ModAPI.CreateParticleEffect("BloodExplosion", transform.position);
			
			NpcBehaviour xnpc = TargetPerson.gameObject.GetOrAddComponent<NpcBehaviour>();
			
			while ( Time.time < Timer1 )
            {
				transform.position = Vector2.Lerp(transform.position,TargetTrans.position, Time.fixedDeltaTime );
				Vector3 LastPos    = transform.position;
				Vector3 zpos       = Vector2.Lerp(transform.position,TargetTrans.position, Time.fixedDeltaTime );
				Vector3 diffpos    = LastPos - zpos;

				transform.position = zpos;
				if ( ChildGadget && ChildGadget.gameObject )
				{
					ChildGadget.MoveChild(diffpos);	
                }
				yield return new WaitForFixedUpdate();

            }
			
			if(gameObject.TryGetComponent<HingeJoint2D>( out HingeJoint2D joint2D)) {
				GameObject.Destroy(joint2D);

				gameObject.GetComponent<Renderer>().enabled = false;
				gameObject.transform.localScale = Vector3.zero;
								
				yield return new WaitForFixedUpdate();
			}

			switch ( Gadget )
			{
				case Gadgets.Memory:
					xnpc.EnhancementMemory = true;
					break;
				case Gadgets.Firearms:
					xnpc.EnhancementFirearms = true;
					break;
				case Gadgets.Karate:
					xnpc.EnhancementKarate = true;
					break;
				case Gadgets.Melee:
					xnpc.EnhancementMelee = true;
					break;
			}

			StopAllCoroutines();
			xnpc.DisableFlip = false;

			Destroy(this);
			if (gameObject) DestroyImmediate(gameObject);
		}

		public IEnumerator IAnimateAIChip()
		{
			Vector3 pos = Vector3.zero;


			for(;;)
			{
				if ( Gadget == Gadgets.AIChip )
                {

					if (PB.charge > 1f ) { Damaged(); }
					if (!BeenActivated) { 
						Lights[0].Radius = 0.225f * Mathf.Cos(Time.frameCount);
						Lights[0].Brightness = 1 + Mathf.Sin(Time.frameCount);
					}
					if (TeamId == 0) { 
						SRS[1].material.SetColor("_Color", Color.HSVToRGB( Mathf.PingPong((Time.time + MiscFloats[0]) * 0.1f, 1), 1, 1));

						Lights[1].Color = Color.HSVToRGB( Mathf.PingPong((Time.time + MiscFloats[0]), 1), 1, 1);
					} else
					{
						SRS[1].material.SetColor("_Color",Color.Lerp(MiscColors[0], MiscColors[1], Mathf.PingPong((Time.time + MiscFloats[0]) , 1)));
					}

					yield return new WaitForEndOfFrame();
					yield return new WaitForFixedUpdate();

				}

				if ( BeenActivated )
				{
					SRS[1].sortingOrder = -1;
					if (!AlreadyAI && TargetPerson.IsTouchingFloor) { 
						RB["UpperBody"].AddForce(new Vector2(0,3) * 170f * Time.fixedDeltaTime);
						TargetPerson.OverridePoseIndex = (int)PoseState.Protective;
					}

					Vector3 LastPos    = transform.position;
					Vector3 zpos       = Vector2.Lerp(transform.position,TargetTrans.position, Time.fixedDeltaTime );
					Vector3 diffpos    = LastPos - zpos;

					transform.position = zpos;


					//if (hinge) hinge.enabled = false;
					//spring.distance *= 0.5f;
					//PB.rigidbody.MovePosition(Vector2.Lerp(transform.position,TargetTrans.position, Time.fixedDeltaTime ));
					//if (hinge) hinge.enabled = true;
					//transform.position = Vector2.Lerp(transform.position,TargetTrans.position, Time.fixedDeltaTime );
					//spring.distance *= 0.99f;
					if ( ChildGadget )
                    {
						//StartCoroutine(ChildGadget.EFixChildAlignment());
						ChildGadget.MoveChild(diffpos);
						//yield return new WaitForFixedUpdate();
                    }


					Lights[0].Color = Lights[1].Color;

					if (AlreadyAI) TargetPerson.OverridePoseIndex = (int)PoseState.WrithingInPain;
					
					pos.x = Mathf.Cos(Time.frameCount);
					pos.y = Mathf.Sin(Time.frameCount);
					float dist1 = AlreadyAI ? xxx.rr(0.1f,1.0f) : 0.23f;
					float dist2 = AlreadyAI ? xxx.rr(0.1f,1.0f) : 0.33f;
					Lights[1].transform.position = transform.position + (pos * dist1); 
					Lights[1].Brightness += Time.fixedDeltaTime * 2;

					pos.y = Mathf.Cos(Time.frameCount);
					pos.x = Mathf.Sin(Time.frameCount);

					Lights[0].transform.position = transform.position + (pos * dist2); 
					Lights[0].Brightness += Time.fixedDeltaTime * 2;

					if (Time.frameCount % 50 == 0) {
						ModAPI.CreateParticleEffect("FuseBlown", transform.position);
						ModAPI.CreateParticleEffect("BloodExplosion", transform.position);
					}

					if ( Time.time > Timer2 && !AlreadyAI ) TargetPerson.OverridePoseIndex = -1;

					if ( Time.time > Timer1 )
					{

						if (Time.time > Timer1 + 2f ) { 

							yield return new WaitForEndOfFrame();

							if(gameObject.TryGetComponent<HingeJoint2D>( out HingeJoint2D joint2D)) {
								GameObject.Destroy(joint2D);
								Lights[0].enabled = Lights[1].enabled = false;
								GameObject.Destroy(Lights[0].gameObject);
								GameObject.Destroy(Lights[1].gameObject);

								gameObject.GetComponent<Renderer>().enabled = false;
								gameObject.transform.localScale = Vector3.zero;
								
								yield return new WaitForFixedUpdate();
							}
							
							if ( AlreadyAI )
							{
								audioSource.PlayOneShot(AudioClips[3], 1f);

								ExplosiveBehaviour explosion = TargetPerson.Limbs[0].gameObject.GetOrAddComponent<ExplosiveBehaviour>();
								explosion.Range              = 5f;
								explosion.ShockwaveStrength  = 1.5f;
								explosion.Delay              = xxx.rr(0.01f,2.0f);
								explosion.FragmentForce      = 2.5f;
								explosion.ShockwaveLiftForce = 1f;
								explosion.BurnPower          = 0f;
								explosion.DismemberChance    = 0.9f;
								explosion.ArmOnAwake         = true;
								explosion.Use(null);
								
								if ( TargetPerson.TryGetComponent<NpcBehaviour>( out NpcBehaviour xnpc ) )
								{
									NpcDeath reaper = xnpc.PBO.gameObject.GetOrAddComponent<NpcDeath>();
									reaper.Config(xnpc);
									GameObject.Destroy(xnpc.FH);
									GameObject.Destroy(xnpc.BH);
									GameObject.Destroy(xnpc);
									//GameObject.Destroy(xnpc.gameObject);
								}
								
								yield return new WaitForSeconds(3);
								

								break;
							}

							foreach( LimbBehaviour limbx in TargetPerson.Limbs ) limbx.Broken = false;
							
							
							audioSource.PlayOneShot(AudioClips[2], 1f);
							yield return new WaitForSeconds(3);
				
							yield return new WaitForFixedUpdate();



							NpcBehaviour npc = TargetPerson.GetComponent<NpcBehaviour>();
							npc.enabled  = true;
							npc.TeamId   = TeamId;
							npc.AIMethod = AIMethods.AIChip;

							


							break;
						}
					}
				}
			}
			
			StopAllCoroutines();
			Destroy(this);
			DestroyImmediate(gameObject);

		}


		

		void OnJointBreak2D (Joint2D brokenJoint) {

			ConnectCoolDown = Time.time + 0.5f;

			if (Gadget == Gadgets.Expansion)
			{
				ModAPI.CreateParticleEffect("Spark", transform.position);
				MiscBools[0] = false;

				xxx.ToggleCollisions(brokenJoint.connectedBody.transform, transform, true);

			} else if (IsChip || Gadget == Gadgets.AIChip)
			{
				ModAPI.CreateParticleEffect("Spark", transform.position);
				xxx.ToggleCollisions(brokenJoint.connectedBody.transform, transform, true);
				if (ChildGadget)
                {
					ChildGadget.ParentGadget = null;
					ChildGadget = null;
                }
			}
		}
		
		public static bool IsAlsoChip( GameObject g )
        {
			if (g && g.TryGetComponent<NpcGadget>( out NpcGadget gadget ) ) return (gadget.IsChip);
			return false;
        }

		public void MoveChild(Vector3 pos)
        {
			transform.position -= pos;
			if (ChildGadget) ChildGadget.MoveChild(pos);
        }

		public IEnumerator EFixChildAlignment()
        {
			if (!ParentGadget || !ParentGadget.gameObject) yield break;
			yield return new WaitForFixedUpdate();
			SpriteRenderer sr2   = gameObject.GetComponent<SpriteRenderer>();
			SpriteRenderer sr	 = ParentGadget.gameObject.GetComponent<SpriteRenderer>();

			float y  = (sr.bounds.max.y  - sr.bounds.min.y);
			float y2 = (sr2.bounds.max.y - sr2.bounds.min.y);

			sr2.sortingOrder = sr.sortingOrder - 1;

			transform.position = ParentGadget.transform.position + new Vector3(0,(y + ((y2-y)/2))-0.022f);
			transform.rotation = ParentGadget.transform.rotation;

			if (ChildGadget) StartCoroutine( ChildGadget.EFixChildAlignment() );
        }

		
		private void OnTriggerEnter2D( Collider2D coll )
        {
			if (coll == null) return;
			if (!coll || !coll.gameObject) return;
			if (!coll.TryGetComponent<NpcGadget>(out _)) return;

			if ((Gadget == Gadgets.AIChip || IsChip) && IsAlsoChip(coll.gameObject) && coll.IsTouching(connector) && Time.time > ConnectCoolDown)
            {
				if (Mathf.Abs(transform.rotation.y) % 360 < 0.1f && Mathf.Abs(coll.gameObject.transform.rotation.z - transform.rotation.z) < 0.1f &&
					Mathf.Abs(coll.gameObject.transform.position.x - transform.position.x) < 0.05f ) { 

					SpriteRenderer sr   = gameObject.GetComponent<SpriteRenderer>();
					SpriteRenderer sr2  = coll.gameObject.GetComponent<SpriteRenderer>();

					ChildGadget              = coll.gameObject.GetComponent<NpcGadget>();
					ChildGadget.ParentGadget = this;

					ModAPI.CreateParticleEffect("Vapor", transform.position);

					xxx.ToggleCollisions(coll.gameObject.transform, transform,false,false);

					float y  = (sr.bounds.max.y  - sr.bounds.min.y);
					float y2 = (sr2.bounds.max.y - sr2.bounds.min.y);

					coll.gameObject.transform.position = transform.position + new Vector3(0,(y + ((y2-y)/2))-0.022f);
					coll.gameObject.transform.rotation = transform.rotation;
					
					FixedJoint2D joint                 = gameObject.AddComponent<FixedJoint2D>();
					joint.connectedBody                = coll.gameObject.GetComponent<Rigidbody2D>();
					joint.autoConfigureConnectedAnchor = true;
					joint.breakForce                   = 1.5f;
					sr2.sortingOrder = sr.sortingOrder - 1;

					StartCoroutine(ChildGadget.EFixChildAlignment());


				}
            }
        }

		private void OnTriggerEnter2D()
        {
			return;
        }

		private void OnCollisionEnter2D(Collision2D coll)
		{
			if (!coll.gameObject) return;
			//if (coll.otherCollider.GetHashCode() == _colliderId) ModAPI.Notify("Edge found");

			//ModAPI.Notify(coll.collider.name);

			if (Gadget == Gadgets.Expansion && coll.gameObject.name == "AI Microchip" && Time.time > ConnectCoolDown)
			{
				if (Mathf.Abs(coll.gameObject.transform.rotation.z - transform.rotation.z) < 0.1f &&
					Mathf.Abs(coll.gameObject.transform.position.x - transform.position.x) < 0.05f ) { 
					
					ModAPI.CreateParticleEffect("Spark", transform.position);

					xxx.ToggleCollisions(coll.gameObject.transform, transform,false,true);

					MiscBools[0]=true;

					
					coll.gameObject.transform.position = transform.position + new Vector3(0,0.1f);
					coll.gameObject.transform.rotation = transform.rotation;
					
					FixedJoint2D joint                 = gameObject.AddComponent<FixedJoint2D>();
					joint.connectedBody                = coll.gameObject.GetComponent<Rigidbody2D>();
					joint.autoConfigureConnectedAnchor = true;
					joint.breakForce                   = 1f;

				}
			}

			

			if (BeenActivated) return;
			if (coll == null || coll.collider == null) return;
			if ( coll.gameObject && coll.transform && coll.transform.root && coll.transform.root.TryGetComponent<PersonBehaviour>( out PersonBehaviour person) )
			{
				if ( person.IsAlive() )
				{
					if (coll.gameObject.TryGetComponent<LimbBehaviour>(out LimbBehaviour lb))
					{ 
						if ( lb.RoughClassification == LimbBehaviour.BodyPart.Head )
						{
							TargetPerson = lb.Person;
							TargetTrans  = lb.transform;

							bool alreadySet = TargetPerson.gameObject.TryGetComponent<NpcBehaviour>(out NpcBehaviour npc);

							if (!alreadySet && Gadget != Gadgets.AIChip) return;

							PhysicalBehaviour pb = gameObject.GetComponent<PhysicalBehaviour>();
							GripBehaviour[] AllGrips = UnityEngine.Object.FindObjectsOfType<GripBehaviour>();
							for ( int i = AllGrips.Length; --i >= 0; )
							{
								if(AllGrips[i].isHolding && AllGrips[i].CurrentlyHolding == pb) AllGrips[i].DropObject();
								if ( AllGrips[i].transform.root.TryGetComponent<NpcBehaviour>( out NpcBehaviour holdingNpc ) )
								{
									holdingNpc.CancelAction = true;
								}
							}
							if (Gadget == Gadgets.AIChip) {
								SRS[1].sortingOrder = -1;
								ModAPI.CreateParticleEffect("FuseBlown", transform.position);
							}
							
							gameObject.GetComponent<SpriteRenderer>().sortingOrder     = -1;
							gameObject.GetComponent<SpriteRenderer>().sortingLayerName = lb.PhysicalBehaviour.spriteRenderer.sortingLayerName;
							
							BeenActivated = true;
							transform.SetParent(lb.transform);

							//	Already has AI
							if ( Gadget == Gadgets.AIChip )
                            {

								if (alreadySet) { 
									AlreadyAI = true;
									ModAPI.CreateParticleEffect("HugeZap", transform.position);
									person.OverridePoseIndex = (int)PoseState.WrithingInPain;
								}
							
								audioSource.PlayOneShot(AudioClips[1], 1f);
							}
							
							//FixedJoint2D joint = gameObject.GetOrAddComponent<FixedJoint2D>();
							Vector3 pp = transform.position;

							//transform.position = TargetTrans.position;
							
							HingeJoint2D xspring = gameObject.GetOrAddComponent<HingeJoint2D>();
							//joint.anchor                       = lb.transform.InverseTransformPoint(transform.position);
							
							xspring.autoConfigureConnectedAnchor = true;
							xspring.connectedBody = lb.gameObject.GetOrAddComponent<Rigidbody2D>( );

							
							//hinge.autoConfigureConnectedAnchor = false;
							//pb.rigidbody.constraints = RigidbodyConstraints2D.FreezeRotation;
							
							gameObject.SetLayer(LayerMask.NameToLayer("Debris"));
							
							
							if ( Gadget == Gadgets.AIChip )
                            {
								if (TeamId          != 0 && MiscColors[0] != null) Lights[1].Color = Lights[0].Color = MiscColors[0];
								else Lights[1].Color = Lights[0].Color = Color.white;
								Lights[0].Brightness = Lights[1].Brightness = AlreadyAI ? 5.5f : 1.5f;
								Lights[0].Radius     = Lights[1].Radius     = 0.125f;
							}

							Timer1 = Time.time + xxx.rr(1f,3f);
							Timer2 = Time.time + xxx.rr(0.1f,1f);

							Rigidbody2D[] RBs   = TargetPerson.transform.GetComponentsInChildren<Rigidbody2D>();
							LimbBehaviour[] LBs = TargetPerson.transform.GetComponentsInChildren<LimbBehaviour>();
							RB.Clear();
							LB.Clear();
							foreach (Rigidbody2D rb in RBs) RB[rb.name] = rb;
							foreach (LimbBehaviour lbx in LBs) LB[lbx.name] =  lbx;

							//LB["LowerArm"].Broken      = true;
							//LB["LowerArmFront"].Broken = true;

							Facing = TargetPerson.transform.localScale.x < 0.0f ? -1f : 1f;
							
							if (alreadySet)
                            {
								npc.DisableFlip = true;
								if (Gadget == Gadgets.AIChip) npc.StartCoroutine(npc.IGlitch());
								else StartCoroutine(IAnimateMiscChip());
                            }  
							else {
								NpcBehaviour xnpc = TargetPerson.gameObject.GetOrAddComponent<NpcBehaviour>();
							}

							
							
						}
					}
				}
			}


			else
			{
				if ( coll.contacts[0].relativeVelocity.magnitude > 12f ) Damaged();
			}

		}



		public void SetupAISerum(PersonBehaviour person)
		{
			TargetPerson = person;

			if ( TargetPerson.IsAlive() )
			{
				BeenActivated = true;

				//	Already has AI
				if (TargetPerson.TryGetComponent<NpcBehaviour>( out NpcBehaviour npc)) { 
					AlreadyAI                = true;
					npc.DisableFlip          = true;
					person.OverridePoseIndex = (int)PoseState.WrithingInPain;
				}

			}

			

			AnimateRoutine = StartCoroutine(IAnimateAISerum());

			Timer1 = Time.time + 4;
		}

		public IEnumerator IAnimateAISerum()
		{
			

			for(;;)
			{
				yield return new WaitForFixedUpdate();
				
				if (Time.time > Timer1 )
				{

					if ( !AlreadyAI )
					{
						ModAPI.CreateParticleEffect("Flash", TargetPerson.Limbs[0].transform.position);
						audioSource.PlayOneShot(NpcMain.AudioSuccess, 0.5f);
						NpcBehaviour npc = TargetPerson.transform.root.gameObject.GetOrAddComponent<NpcBehaviour>();
						yield return new WaitForSeconds( 3 );
						yield return new WaitForFixedUpdate();

						npc.enabled = true;
						npc.TeamId = TeamId;
						npc.AIMethod = AIMethods.Syringe;
						StopAllCoroutines();
						Destroy( this );
					} else
					{
						NpcBehaviour npcx = TargetPerson.transform.root.gameObject.GetOrAddComponent<NpcBehaviour>();
						if (npcx.AIMethod == AIMethods.AIChip) npcx.StartCoroutine(npcx.IGlitch());
					}
					break;
				}


			}

			

		}
	}


	public class AISyringe : SyringeBehaviour
	{
		public override string GetLiquidID() => AISerum.ID;
		public class AISerum : Liquid
		{
			public const string ID = "AI SERUM";

			public AISerum()
			{
				Color = new UnityEngine.Color(0.733f, 0.211f, 0.027f);
			}

			public override void OnEnterLimb(LimbBehaviour limb)
			{
				NpcGadget gadget = limb.Person.gameObject.GetOrAddComponent<NpcGadget>();
				gadget.SetupAISerum(limb.Person);
			}

			public override void OnUpdate(BloodContainer container) {

			}
			public override void OnEnterContainer(BloodContainer container) {
					//var glow = ModAPI.CreateLight(container.transform, Color.red, 5, 1);
			}
			public override void OnExitContainer(BloodContainer container) {}
			
		}
	}
}
